"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const component_1 = require("../common/component/component");
class Page extends component_1.Component {
    constructor(geneticCode) {
        super(Object.assign({ className: 'Page', tag: 'page' }, geneticCode));
        this.unknown = false;
        if (geneticCode.file)
            serviceModel_1.ServiceModel.getPromise(geneticCode.file + 'L').then((data) => this.checkLanguage(geneticCode.file, data)).fail((data) => this.checkFailed(data));
    }
    getLanguageJSON() {
        return this.language;
    }
    getCurrentLanguageJSON() {
        let jSON = this.language;
        let property;
        for (property in jSON) {
            if (property !== undefined) {
                if (!jSON.hasOwnProperty(property)) {
                    continue;
                }
                if (jSON[property]['language'] === this.getCurrentLanguage()) {
                    // console.log('LANG:'+jSON[property]['language']);
                    break;
                }
            }
        }
        return jSON[property];
    }
    getUnknown() {
        return this.unknown;
    }
    setUnknown(unknown) {
        this.unknown = unknown;
    }
    checkLanguage(file, jSON) {
        this.language = jSON;
        serviceModel_1.ServiceModel.getPromise(file).then((data) => this.checkArray(data)).fail((data) => this.checkFailed(data));
    }
    checkArray(jSON) {
        // jSON = JSON.parse(jSON);
        if (jSON.constructor === Array) {
            for (let index = 0; index < jSON.length; index++) {
                let fJSON = {
                    minWidth: jSON[index]['minWidth'],
                    maxWidth: jSON[index]['maxWidth'],
                    minHeight: jSON[index]['minHeight'],
                    maxHeight: jSON[index]['maxHeight'],
                };
                serviceModel_1.ServiceModel.getPromise(jSON[index]['file']).then((data) => this.setCurrentPage(data, fJSON)).fail((data) => this.checkFailed(data));
            }
        }
        else {
            this.setCurrentPage(jSON);
        }
    }
    checkFailed(data) {
        // console.error('JSONT:' + data);
        if (this.father instanceof componentRouter_1.ComponentRouter)
            this.father.updateFailed(data, this);
    }
    setCurrentPage(jSON, fJSON) {
        let frame = this.newFrame(fJSON);
        let refresh = this.checkSize(frame);
        frame.populate(jSON);
        if (refresh) {
            this.refreshFrame(frame);
        }
    }
    checkFrame(frame) {
        return ((this.father instanceof componentRouter_1.ComponentRouter) && (this.currentFrame === undefined || this.currentFrame !== frame || this.father.toGo()));
    }
    checkSize(frame) {
        let refresh = false;
        if (this.checkFrame(frame) && this.checkHeight(frame) && this.checkWidth(frame)) {
            this.father.went();
            refresh = true;
        }
        return refresh;
    }
    checkHeight(frame) {
        let height = document.documentElement.clientHeight;
        return ((frame.getMaxHeight() === undefined || frame.getMaxHeight() >= height) && (frame.getMinHeight() === undefined || frame.getMinHeight() <= height));
    }
    checkWidth(frame) {
        let width = document.documentElement.clientWidth;
        return ((frame.getMaxWidth() === undefined || frame.getMaxWidth() >= width) && (frame.getMinWidth() === undefined || frame.getMinWidth() <= width));
    }
    refreshFrame(frame) {
        this.currentFrame = frame;
        this.destroyChildElements();
        this.insert(frame);
        this.renderAfterUpdate();
    }
    newFrame(fJSON) {
        let frame = new componentPageFrame_1.ComponentPageFrame({ father: this });
        if (fJSON) {
            frame.setMinWidth(fJSON.minWidth);
            frame.setMaxWidth(fJSON.maxWidth);
            frame.setMinHeight(fJSON.minHeight);
            frame.setMaxHeight(fJSON.maxHeight);
        }
        return frame;
    }
    addFrame(frame, jSON) {
        if (this.getArrayChild.indexOf(frame) === -1) {
            this.addChild({ child: frame });
            frame.populate(jSON);
        }
    }
}
exports.Page = Page;
const componentPageFrame_1 = require("./componentPageFrame");
const serviceModel_1 = require("../serviceModel/serviceModel");
const componentRouter_1 = require("../common/component/generic/router/componentRouter");
//# sourceMappingURL=page.js.map